/*
 * Vector.hpp
 *
 * template class representing general vector in some
 * space (determined by point type class).
 *
 */
#ifndef INCLUDE_VECTORS_VECTOR_HPP_FILE
#define INCLUDE_VECTORS_VECTOR_HPP_FILE

/* public header */

#include <math.h>
#include <boost/operators.hpp>
#include <assert.h>

#include "Vectors/Point.hpp"
#include "Vectors/Exception.hpp"


namespace Vectors
{

// we implement Vector as Point in equivalen space
template <typename TPoint>
class Vector: public boost::equality_comparable< Vector<TPoint> >
{
public:
  Vector(void)
  {
  }

  // make vector p1 -> p2
  Vector(const TPoint &p1, const TPoint &p2)
  {
    unsigned int dims=p1.size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]=p2[i]-p1[i];
  }

  // distance from [0...0] -> *this
  inline double length(void) const
  {
    TPoint       pnt;
    unsigned int dims=pnt.size();
    for(unsigned int i=0; i<dims; ++i)
      pnt[i]=0;

    return _p.dist(pnt);
  }

  // adds two Vectors
  inline Vectors::Vector<TPoint> &operator+=(const Vectors::Vector<TPoint> &v)
  {
    unsigned int dims=size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]+=v[i];

    return *this;
  }

  // multiply Vector by some value (resize vector)
  inline Vectors::Vector<TPoint> &operator*=(double c)
  {
    unsigned int dims=size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]*=c;

    return *this;
  }
  // multiply Vector by some value (resize vector)
  inline Vectors::Vector<TPoint> &operator*=(int c)
  {
    unsigned int dims=size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]*=c;

    return *this;
  }

  // comparison
  inline bool operator==( const Vectors::Vector<TPoint> &v) const
  {
    return _p==v._p;
  }

  // scalar multiply of a vectors
  inline double operator*(const Vectors::Vector<TPoint> &v) const
  {
    unsigned int dims=size();
    double       res =0;
    for(unsigned int i=0; i<dims; ++i)
      res+=(*this)[i]*v[i];

    return res;
  }

  // normalizes Vector
  inline void normalize(void)
  {
    const double len=length();
    if(len==0)
      throw Vectors::Exception("Vector::normalize(): division by zero");

    unsigned int dims=size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]/=len;
  }

  // reverse direction of vector
  inline void reverse(void)
  {
    unsigned int dims=size();
    for(unsigned int i=0; i<dims; ++i)
      (*this)[i]*=-1;
  }

  // get size of vector (number of dimentions)
  inline unsigned int size(void) const
  {
    return _p.size();
  }

  // get dimention field
  inline typename TPoint::TThisDimType &operator[](unsigned int pos)
  {
    return _p[pos];
  }

  // get dimention field
  inline const typename TPoint::TThisDimType &operator[](unsigned int pos) const
  {
    return _p[pos];
  }

private:
  TPoint _p;
}; // class Vector


//
// some commolny used spaces type
//

typedef Vector<Point2DCont> Vector2DCont;
typedef Vector<Point2DDisc> Vector2DDisc;

typedef Vector<Point3DCont> Vector3DCont;
typedef Vector<Point3DDisc> Vector3DDisc;

}; // namespace Vectors

#endif

